=== modified file 'drizzled/cursor.h'
--- drizzled/cursor.h	2009-12-01 00:32:58 +0000
+++ drizzled/cursor.h	2009-12-01 01:54:04 +0000
@@ -562,24 +562,6 @@
   virtual int final_drop_index(Table *)
   { return (HA_ERR_WRONG_COMMAND); }
 
-  uint32_t max_record_length() const
-  { return std::min((unsigned int)HA_MAX_REC_LENGTH, max_supported_record_length()); }
-  uint32_t max_keys() const
-  { return std::min((unsigned int)MAX_KEY, max_supported_keys()); }
-  uint32_t max_key_parts() const
-  { return std::min((unsigned int)MAX_REF_PARTS, max_supported_key_parts()); }
-  uint32_t max_key_length() const
-  { return std::min((unsigned int)MAX_KEY_LENGTH, max_supported_key_length()); }
-  uint32_t max_key_part_length(void) const
-  { return std::min((unsigned int)MAX_KEY_LENGTH, max_supported_key_part_length()); }
-
-  virtual uint32_t max_supported_record_length(void) const
-  { return HA_MAX_REC_LENGTH; }
-  virtual uint32_t max_supported_keys(void) const { return 0; }
-  virtual uint32_t max_supported_key_parts(void) const { return MAX_REF_PARTS; }
-  virtual uint32_t max_supported_key_length(void) const { return MAX_KEY_LENGTH; }
-  virtual uint32_t max_supported_key_part_length(void) const { return 255; }
-
   virtual bool low_byte_first(void) const { return true; }
   virtual uint32_t checksum(void) const { return 0; }
   virtual bool is_crashed(void) const  { return false; }

=== modified file 'drizzled/item/subselect.cc'
--- drizzled/item/subselect.cc	2009-11-19 18:58:35 +0000
+++ drizzled/item/subselect.cc	2009-12-01 01:54:04 +0000
@@ -2964,8 +2964,8 @@
     assert(tmp_table->s->db_type() == myisam_engine);
     assert(
       tmp_table->s->uniques ||
-      tmp_table->key_info->key_length >= tmp_table->cursor->max_key_length() ||
-      tmp_table->key_info->key_parts > tmp_table->cursor->max_key_parts());
+      tmp_table->key_info->key_length >= tmp_table->cursor->getEngine()->max_key_length() ||
+      tmp_table->key_info->key_parts > tmp_table->cursor->getEngine()->max_key_parts());
     tmp_table->free_tmp_table(session);
     delete result;
     result= NULL;

=== modified file 'drizzled/plugin/storage_engine.h'
--- drizzled/plugin/storage_engine.h	2009-12-01 00:32:58 +0000
+++ drizzled/plugin/storage_engine.h	2009-12-01 01:54:04 +0000
@@ -417,6 +417,25 @@
   static void removeLostTemporaryTables(Session &session, const char *directory);
 
   Cursor *getCursor(TableShare &share, MEM_ROOT *alloc);
+
+  uint32_t max_record_length() const
+  { return std::min((unsigned int)HA_MAX_REC_LENGTH, max_supported_record_length()); }
+  uint32_t max_keys() const
+  { return std::min((unsigned int)MAX_KEY, max_supported_keys()); }
+  uint32_t max_key_parts() const
+  { return std::min((unsigned int)MAX_REF_PARTS, max_supported_key_parts()); }
+  uint32_t max_key_length() const
+  { return std::min((unsigned int)MAX_KEY_LENGTH, max_supported_key_length()); }
+  uint32_t max_key_part_length(void) const
+  { return std::min((unsigned int)MAX_KEY_LENGTH, max_supported_key_part_length()); }
+
+  virtual uint32_t max_supported_record_length(void) const
+  { return HA_MAX_REC_LENGTH; }
+  virtual uint32_t max_supported_keys(void) const { return 0; }
+  virtual uint32_t max_supported_key_parts(void) const { return MAX_REF_PARTS; }
+  virtual uint32_t max_supported_key_length(void) const { return MAX_KEY_LENGTH; }
+  virtual uint32_t max_supported_key_part_length(void) const { return 255; }
+
 };
 
 } /* namespace plugin */

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2009-12-01 00:32:58 +0000
+++ drizzled/sql_table.cc	2009-12-01 01:54:04 +0000
@@ -793,7 +793,7 @@
 
   select_field_pos= alter_info->create_list.elements - select_field_count;
   null_fields=blob_columns=0;
-  max_key_length= cursor->max_key_length();
+  max_key_length= cursor->getEngine()->max_key_length();
 
   for (field_no=0; (sql_field=it++) ; field_no++)
   {
@@ -1082,7 +1082,7 @@
       continue;
     }
     (*key_count)++;
-    tmp=cursor->max_key_parts();
+    tmp=cursor->getEngine()->max_key_parts();
     if (key->columns.elements > tmp)
     {
       my_error(ER_TOO_MANY_KEY_PARTS,MYF(0),tmp);
@@ -1131,7 +1131,7 @@
       return(true);
     }
   }
-  tmp=cursor->max_keys();
+  tmp=cursor->getEngine()->max_keys();
   if (*key_count > tmp)
   {
     my_error(ER_TOO_MANY_KEYS,MYF(0),tmp);
@@ -1308,9 +1308,9 @@
 	if (sql_field->sql_type == DRIZZLE_TYPE_BLOB)
 	{
 	  if ((length=column->length) > max_key_length ||
-	      length > cursor->max_key_part_length())
+	      length > cursor->getEngine()->max_key_part_length())
 	  {
-	    length= min(max_key_length, cursor->max_key_part_length());
+	    length= min(max_key_length, cursor->getEngine()->max_key_part_length());
 	    if (key->type == Key::MULTIPLE)
 	    {
 	      /* not a critical problem */
@@ -1345,9 +1345,9 @@
 	my_error(ER_WRONG_KEY_COLUMN, MYF(0), column->field_name.str);
 	  return(true);
       }
-      if (length > cursor->max_key_part_length())
+      if (length > cursor->getEngine()->max_key_part_length())
       {
-        length= cursor->max_key_part_length();
+        length= cursor->getEngine()->max_key_part_length();
 	if (key->type == Key::MULTIPLE)
 	{
 	  /* not a critical problem */

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2009-11-30 23:49:31 +0000
+++ drizzled/table.cc	2009-12-01 01:54:04 +0000
@@ -2633,8 +2633,8 @@
     share->storage_engine= myisam_engine;
     table->cursor= share->db_type()->getCursor(*share, &table->mem_root);
     if (group &&
-	(param->group_parts > table->cursor->max_key_parts() ||
-	 param->group_length > table->cursor->max_key_length()))
+	(param->group_parts > table->cursor->getEngine()->max_key_parts() ||
+	 param->group_length > table->cursor->getEngine()->max_key_length()))
       using_unique_constraint=1;
   }
   else
@@ -3181,8 +3181,8 @@
       goto err;
 
     memset(seg, 0, sizeof(*seg) * keyinfo->key_parts);
-    if (keyinfo->key_length >= cursor->max_key_length() ||
-	keyinfo->key_parts > cursor->max_key_parts() ||
+    if (keyinfo->key_length >= cursor->getEngine()->max_key_length() ||
+	keyinfo->key_parts > cursor->getEngine()->max_key_parts() ||
 	share->uniques)
     {
       /* Can't create a key; Make a unique constraint instead of a key */

=== modified file 'plugin/archive/ha_archive.cc'
--- plugin/archive/ha_archive.cc	2009-11-30 23:49:31 +0000
+++ plugin/archive/ha_archive.cc	2009-12-01 01:54:04 +0000
@@ -175,6 +175,10 @@
   ArchiveShare *findOpenTable(const string table_name);
   void addOpenTable(const string &table_name, ArchiveShare *);
   void deleteOpenTable(const string &table_name);
+
+  uint32_t max_supported_keys()          const { return 1; }
+  uint32_t max_supported_key_length()    const { return sizeof(uint64_t); }
+  uint32_t max_supported_key_part_length() const { return sizeof(uint64_t); }
 };
 
 

=== modified file 'plugin/archive/ha_archive.h'
--- plugin/archive/ha_archive.h	2009-11-26 02:37:40 +0000
+++ plugin/archive/ha_archive.h	2009-12-01 01:54:04 +0000
@@ -102,9 +102,6 @@
   }
   void get_auto_increment(uint64_t, uint64_t, uint64_t,
                           uint64_t *first_value, uint64_t *nb_reserved_values);
-  uint32_t max_supported_keys()          const { return 1; }
-  uint32_t max_supported_key_length()    const { return sizeof(uint64_t); }
-  uint32_t max_supported_key_part_length() const { return sizeof(uint64_t); }
   ha_rows records() { return share->rows_recorded; }
   int index_init(uint32_t keynr, bool sorted);
   virtual int index_read(unsigned char * buf, const unsigned char * key,

=== modified file 'plugin/blackhole/ha_blackhole.cc'
--- plugin/blackhole/ha_blackhole.cc	2009-11-30 23:49:31 +0000
+++ plugin/blackhole/ha_blackhole.cc	2009-12-01 01:54:04 +0000
@@ -78,7 +78,7 @@
   {
     CachedDirectory::Entries entries= directory.getEntries();
 
-    for (CachedDirectory::Entries::iterator entry_iter= entries.begin(); 
+    for (CachedDirectory::Entries::iterator entry_iter= entries.begin();
          entry_iter != entries.end(); ++entry_iter)
     {
       CachedDirectory::Entry *entry= *entry_iter;
@@ -98,11 +98,16 @@
 
         file_name_len= filename_to_tablename(filename->c_str(), uname, sizeof(uname));
         // TODO: Remove need for memory copy here
-        uname[file_name_len - sizeof(BLACKHOLE_EXT) + 1]= '\0'; // Subtract ending, place NULL 
+        uname[file_name_len - sizeof(BLACKHOLE_EXT) + 1]= '\0'; // Subtract ending, place NULL
         set_of_names.insert(uname);
       }
     }
   }
+
+  /* The following defines can be increased if necessary */
+  uint32_t max_supported_keys()          const { return BLACKHOLE_MAX_KEY; }
+  uint32_t max_supported_key_length()    const { return BLACKHOLE_MAX_KEY_LENGTH; }
+  uint32_t max_supported_key_part_length() const { return BLACKHOLE_MAX_KEY_LENGTH; }
 };
 
 /* Static declarations for shared structures */

=== modified file 'plugin/blackhole/ha_blackhole.h'
--- plugin/blackhole/ha_blackhole.h	2009-11-16 20:30:19 +0000
+++ plugin/blackhole/ha_blackhole.h	2009-12-01 01:54:04 +0000
@@ -54,10 +54,6 @@
   */
   const char *index_type(uint32_t key_number);
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
-  /* The following defines can be increased if necessary */
-  uint32_t max_supported_keys()          const { return BLACKHOLE_MAX_KEY; }
-  uint32_t max_supported_key_length()    const { return BLACKHOLE_MAX_KEY_LENGTH; }
-  uint32_t max_supported_key_part_length() const { return BLACKHOLE_MAX_KEY_LENGTH; }
   int open(const char *name, int mode, uint32_t test_if_locked);
   int close(void);
   int write_row(unsigned char * buf);

=== modified file 'plugin/csv/ha_tina.cc'
--- plugin/csv/ha_tina.cc	2009-11-30 23:49:31 +0000
+++ plugin/csv/ha_tina.cc	2009-12-01 01:54:04 +0000
@@ -152,6 +152,10 @@
   void doGetTableNames(CachedDirectory &, string& , set<string>&) { };
 
   int doDropTable(Session&, const string table_path);
+
+  uint32_t max_keys()          const { return 0; }
+  uint32_t max_key_parts()     const { return 0; }
+  uint32_t max_key_length()    const { return 0; }
 };
 
 int Tina::doDropTable(Session&,

=== modified file 'plugin/csv/ha_tina.h'
--- plugin/csv/ha_tina.h	2009-11-22 05:25:48 +0000
+++ plugin/csv/ha_tina.h	2009-12-01 01:54:04 +0000
@@ -109,10 +109,6 @@
     */
     return 0;
   }
-  uint32_t max_record_length() const { return HA_MAX_REC_LENGTH; }
-  uint32_t max_keys()          const { return 0; }
-  uint32_t max_key_parts()     const { return 0; }
-  uint32_t max_key_length()    const { return 0; }
   /*
      Called in test_quick_select to determine if indexes should be used.
    */

=== modified file 'plugin/heap/ha_heap.cc'
--- plugin/heap/ha_heap.cc	2009-11-30 23:49:31 +0000
+++ plugin/heap/ha_heap.cc	2009-12-01 01:54:04 +0000
@@ -91,6 +91,8 @@
   /* Temp only engine, so do not return values. */
   void doGetTableNames(CachedDirectory &, string& , set<string>&) { };
 
+  uint32_t max_supported_keys()          const { return MAX_KEY; }
+  uint32_t max_supported_key_part_length() const { return MAX_KEY_LENGTH; }
 };
 
 int HeapEngine::doGetTableDefinition(Session&,

=== modified file 'plugin/heap/ha_heap.h'
--- plugin/heap/ha_heap.h	2009-11-22 05:25:48 +0000
+++ plugin/heap/ha_heap.h	2009-12-01 01:54:04 +0000
@@ -44,8 +44,6 @@
   enum row_type get_row_type() const;
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
   const key_map *keys_to_use_for_scanning() { return &btree_keys; }
-  uint32_t max_supported_keys()          const { return MAX_KEY; }
-  uint32_t max_supported_key_part_length() const { return MAX_KEY_LENGTH; }
   double scan_time()
   { return (double) (stats.records+stats.deleted) / 20.0+10; }
   double read_time(uint32_t, uint32_t,

=== modified file 'plugin/innobase/handler/ha_innodb.cc'
--- plugin/innobase/handler/ha_innodb.cc	2009-11-30 23:49:31 +0000
+++ plugin/innobase/handler/ha_innodb.cc	2009-12-01 01:54:04 +0000
@@ -387,6 +387,9 @@
 
   UNIV_INTERN virtual bool get_error_message(int error, String *buf);
 
+  UNIV_INTERN uint32_t max_supported_keys() const;
+  UNIV_INTERN uint32_t max_supported_key_length() const;
+  UNIV_INTERN uint32_t max_supported_key_part_length() const;
 };
 
 /** @brief Initialize the default value of innodb_commit_concurrency.
@@ -2529,7 +2532,7 @@
 @return	MAX_KEY */
 UNIV_INTERN
 uint
-ha_innobase::max_supported_keys() const
+InnobaseEngine::max_supported_keys() const
 /*===================================*/
 {
 	return(MAX_KEY);
@@ -2540,7 +2543,7 @@
 @return	maximum supported key length, in bytes */
 UNIV_INTERN
 uint32_t
-ha_innobase::max_supported_key_length() const
+InnobaseEngine::max_supported_key_length() const
 /*=========================================*/
 {
 	/* An InnoDB page must store >= 2 keys; a secondary key record
@@ -2899,7 +2902,7 @@
 
 UNIV_INTERN
 uint32_t
-ha_innobase::max_supported_key_part_length() const
+InnobaseEngine::max_supported_key_part_length() const
 {
 	return(DICT_MAX_INDEX_COL_LEN - 1);
 }

=== modified file 'plugin/innobase/handler/ha_innodb.h'
--- plugin/innobase/handler/ha_innodb.h	2009-11-26 02:37:40 +0000
+++ plugin/innobase/handler/ha_innodb.h	2009-12-01 01:54:04 +0000
@@ -109,9 +109,6 @@
 
 	UNIV_INTERN const char* index_type(uint key_number);
 	UNIV_INTERN uint32_t index_flags(uint idx, uint part, bool all_parts) const;
-	UNIV_INTERN uint32_t max_supported_keys() const;
-	UNIV_INTERN uint32_t max_supported_key_length() const;
-	UNIV_INTERN uint32_t max_supported_key_part_length() const;
 	UNIV_INTERN const key_map* keys_to_use_for_scanning();
 
 	UNIV_INTERN int open(const char *name, int mode, uint test_if_locked);

=== modified file 'plugin/myisam/ha_myisam.cc'
--- plugin/myisam/ha_myisam.cc	2009-11-30 23:49:31 +0000
+++ plugin/myisam/ha_myisam.cc	2009-12-01 01:54:04 +0000
@@ -104,6 +104,9 @@
   /* Temp only engine, so do not return values. */
   void doGetTableNames(CachedDirectory &, string& , set<string>&) { };
 
+  uint32_t max_supported_keys()          const { return MI_MAX_KEY; }
+  uint32_t max_supported_key_length()    const { return MI_MAX_KEY_LENGTH; }
+  uint32_t max_supported_key_part_length() const { return MI_MAX_KEY_LENGTH; }
 };
 
 int MyisamEngine::doGetTableDefinition(Session&,

=== modified file 'plugin/myisam/ha_myisam.h'
--- plugin/myisam/ha_myisam.h	2009-11-22 05:25:48 +0000
+++ plugin/myisam/ha_myisam.h	2009-12-01 01:54:04 +0000
@@ -48,9 +48,6 @@
   int index_init(uint32_t idx, bool sorted);
   int index_end();
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
-  uint32_t max_supported_keys()          const { return MI_MAX_KEY; }
-  uint32_t max_supported_key_length()    const { return MI_MAX_KEY_LENGTH; }
-  uint32_t max_supported_key_part_length() const { return MI_MAX_KEY_LENGTH; }
   uint32_t checksum() const;
 
   int open(const char *name, int mode, uint32_t test_if_locked);

